using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.WindowsMobile.DirectX;
using Microsoft.WindowsMobile.DirectX.Direct3D;


namespace Snake3D
{
    public partial class Snake3DForm : Form
    {
        #region Drawing variables

        /// <summary>
        /// Direct3D device we are going to draw on.
        /// </summary>
        Device device;

        #endregion

        #region Game Objects

        /// <summary>
        /// Keeps track of the position of a segment of the snake.
        /// </summary>
        class SnakeSegment
        {
            /// <summary>
            /// Speed of the segment in the X direction.
            /// </summary>
            public float XSpeed;

            /// <summary>
            /// Speed of the segment in the X direction.
            /// </summary>
            public float YSpeed;

            /// <summary>
            /// Speed of the segment in the Z direction
            /// </summary>
            public float ZSpeed;

            /// <summary>
            /// Draws the segment on the given graphics device
            /// </summary>
            /// <param name="g">device to be used for drawing</param>
            public void Update()
            {
                for (int i = 0 ; i < 6 ; i++)
                {
                    Vertices[i].X += XSpeed;
                    Vertices[i].Y += YSpeed;
                    Vertices[i].Z += ZSpeed;
                }
            }

            public void PlaceInZ ( float X, float Y, float Z, float width, float height ) 
            {
                Vertices[0].X = X;
                Vertices[0].Y = Y;
                Vertices[0].Z = Z;

                Vertices[1].X = X;
                Vertices[1].Y = Y+height;
                Vertices[1].Z = Z;

                Vertices[2].X = X+width;
                Vertices[2].Y = Y;
                Vertices[2].Z = Z;

                Vertices[3].X = X+width;
                Vertices[3].Y = Y;
                Vertices[3].Z = Z;

                Vertices[4].X = X;
                Vertices[4].Y = Y+height;
                Vertices[4].Z = Z;

                Vertices[5].X = X+width;
                Vertices[5].Y = Y+height;
                Vertices[5].Z = Z;

            }

            /// <summary>
            /// We make the segment out of two triangles, 
            /// specified by 6 vertices.
            /// </summary>
            public CustomVertex.PositionColored[] Vertices ;

            /// <summary>
            /// Vertex buffer which is used by the display hardware 
            /// recieve details of what we want to draw.
            /// </summary>
            public VertexBuffer VertBuffer;

            /// <summary>
            /// Constructor. Do not set the position, since that
            /// will be set when the snake is placed on the screen.
            /// </summary>
            /// <param name="inBrush">brush to draw the snake segment with.</param>
            public SnakeSegment( Color inSegmentColour, Device device )
            {
                int colour = inSegmentColour.ToArgb(); ;

                Vertices = new CustomVertex.PositionColored[6];

                /// Set the color of the vertices.
                for (int i = 0; i < Vertices.Length; i++)
                {
                    Vertices[i].Color = colour;
                }

                /// Create a vertex buffer for our segment
                VertBuffer = new VertexBuffer(
                    typeof(CustomVertex.PositionColored),   // type of the buffer
                    6,                                      // holding 3 vertices
                    device,                                 // for our device
                    Usage.WriteOnly,                        // never going to read it
                    CustomVertex.PositionColored.Format,    // source format
                    Pool.SystemMemory);                     // mobile 3D requires this
            }

            public void Draw(Device device)
            {
                // Set the data in the vertex buffer to our triangles
                VertBuffer.SetData(Vertices, 0, LockFlags.None);

                // Point the device at our vertex buffer
                device.SetStreamSource(0, VertBuffer, 0);

                // Ask the device to draw the contents of the buffer
                device.DrawPrimitives(PrimitiveType.TriangleList, 0, 2);
            }
        }

        private SnakeSegment segment;

        #endregion

        public Snake3DForm()
        {
            InitializeComponent();
        }

        #region Game Management


        public void Init()
        {
            PresentParameters presentParams = new PresentParameters();
            presentParams.Windowed = true;
            presentParams.SwapEffect = SwapEffect.Discard;
            device = new Device(
                0,                      // device number 0
                DeviceType.Default,     // default configuration
                this,                   // reference to the parent window
                CreateFlags.None,       // no special creation flags
                presentParams);         // the presentation parameters

            device.RenderState.Lighting = false;

            segment = new SnakeSegment(Color.Red, device);

            segment.PlaceInZ(0, 0, 0, .2f, .3f);

        }

        private void Render()
        {
            device.Clear(ClearFlags.Target, Color.White, 1.0f, 0);
            device.BeginScene();

            segment.Draw(device);

            device.EndScene();
            device.Present();
        }

        #endregion

        #region Event Handlers

        protected override void OnPaintBackground(PaintEventArgs e)
        {
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            Render();
        }

        #endregion 
    }
}