using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.WindowsMobile.DirectX;
using Microsoft.WindowsMobile.DirectX.Direct3D;


namespace Snake3D
{
    public partial class Snake3DForm : Form
    {
        #region Drawing variables

        /// <summary>
        /// Direct3D device we are going to draw on.
        /// </summary>
        Device device;

        #endregion

        #region Game Objects

        /// <summary>
        /// Keeps track of the position of a segment of the snake.
        /// </summary>
        class SnakeSegment
        {
            /// <summary>
            /// Speed of the segment in the X direction.
            /// </summary>
            public float XSpeed;

            /// <summary>
            /// Speed of the segment in the X direction.
            /// </summary>
            public float YSpeed;

            /// <summary>
            /// Speed of the segment in the Z direction
            /// </summary>
            public float ZSpeed;

            /// <summary>
            /// Draws the segment on the given graphics device
            /// </summary>
            /// <param name="g">device to be used for drawing</param>
            public void Update()
            {
                for (int i = 0 ; i < 6 ; i++)
                {
                    Vertices[i].X += XSpeed;
                    Vertices[i].Y += YSpeed;
                    Vertices[i].Z += ZSpeed;
                }
            }

            public void PlaceInZ ( float X, float Y, float Z, float width, float height ) 
            {
                Vertices[0].X = X;
                Vertices[0].Y = Y;
                Vertices[0].Z = Z;

                Vertices[1].X = X;
                Vertices[1].Y = Y+height;
                Vertices[1].Z = Z;

                Vertices[2].X = X+width;
                Vertices[2].Y = Y;
                Vertices[2].Z = Z;

                Vertices[3].X = X+width;
                Vertices[3].Y = Y;
                Vertices[3].Z = Z;

                Vertices[4].X = X;
                Vertices[4].Y = Y+height;
                Vertices[4].Z = Z;

                Vertices[5].X = X+width;
                Vertices[5].Y = Y+height;
                Vertices[5].Z = Z;

            }

            /// <summary>
            /// We make the segment out of two triangles, 
            /// specified by 6 vertices.
            /// </summary>
            public CustomVertex.PositionColored[] Vertices ;

            /// <summary>
            /// Vertex buffer which is used by the display hardware 
            /// recieve details of what we want to draw.
            /// </summary>
            public VertexBuffer VertBuffer;

            /// <summary>
            /// Constructor. Do not set the position, since that
            /// will be set when the snake is placed on the screen.
            /// </summary>
            /// <param name="inBrush">brush to draw the snake segment with.</param>
            public SnakeSegment( Color inSegmentColour, Device device )
            {
                int colour = inSegmentColour.ToArgb(); ;

                Vertices = new CustomVertex.PositionColored[6];

                /// Set the color of the vertices.
                for (int i = 0; i < Vertices.Length; i++)
                {
                    Vertices[i].Color = colour;
                }

                /// Create a vertex buffer for our segment
                VertBuffer = new VertexBuffer(
                    typeof(CustomVertex.PositionColored),   // type of the buffer
                    6,                                      // holding 3 vertices
                    device,                                 // for our device
                    Usage.WriteOnly,                        // never going to read it
                    CustomVertex.PositionColored.Format,    // source format
                    Pool.SystemMemory);                     // mobile 3D requires this

            }

            public void Draw(Device device)
            {
                // Set the data in the vertex buffer to our triangles
                VertBuffer.SetData(Vertices, 0, LockFlags.None);

                // Point the device at our vertex buffer
                device.SetStreamSource(0, VertBuffer, 0);

                // Ask the device to draw the contents of the buffer
                device.DrawPrimitives(PrimitiveType.TriangleList, 0, 2);
            }
        }

        private SnakeSegment segment1;
        private SnakeSegment segment2;

        #endregion

        public Snake3DForm()
        {
            InitializeComponent();
        }

        #region Timer routines

        /// <summary>
        /// The tick count recorded at the start of the timing period.
        /// </summary>
        private int clockStartTick;

        /// <summary>
        /// Counter for the number of readings to be taken.
        /// </summary>
        private int readingCount = 0;

        /// <summary>
        /// Number of readings to be averaged over.
        /// </summary>
        private int sampleSize = 20;

        /// <summary>
        /// Total of readings taken.
        /// </summary>
        private float readingTotal = 0.0f;

        /// <summary>
        /// Calculated average result.
        /// </summary>
        private float timerResult;

        /// <summary>
        /// Starts the timer ticking.
        /// </summary>
        private void startTimer()
        {
            clockStartTick = System.Environment.TickCount;
        }

        /// <summary>
        /// Obtains a new time value, adds it to the total and
        /// displays the average if we are at the end of a sample
        /// period.
        /// </summary>
        private void showTimer()
        {
            readingTotal += (System.Environment.TickCount - clockStartTick);

            if (++readingCount == sampleSize)
            {
                timerResult = readingTotal / sampleSize;
                this.Text = "Time : " + timerResult.ToString();
                readingTotal = 0.0f;
                readingCount = 0;
            }
        }

        #endregion

        #region Game Management

        #region Game State variables

        /// <summary>
        /// When true the game is being played.
        /// </summary>
        private bool gameActive = false;

        /// <summary>
        /// When true the game is in pause state.
        /// </summary>
        private bool gamePaused = false;

        #endregion

        public void Init()
        {
            PresentParameters presentParams = new PresentParameters();
            presentParams.Windowed = true;
            presentParams.SwapEffect = SwapEffect.Discard;
            device = new Device(
                0,                      // device number 0
                DeviceType.Default,     // default configuration
                this,                   // reference to the parent window
                CreateFlags.None,       // no special creation flags
                presentParams);         // the presentation parameters

            device.RenderState.Lighting = false;

            segment1 = new SnakeSegment(Color.Red, device);

            segment1.PlaceInZ(0, 0, 0, .2f, .3f);

            segment1.XSpeed = 0.01f;
            segment1.YSpeed = 0.01f;
            segment1.ZSpeed = 0.0f;

            segment2 = new SnakeSegment(Color.Green, device);

            segment2.PlaceInZ(-.2f, -.3f, 0, .2f, .3f);

            segment2.XSpeed = -0.01f;
            segment2.YSpeed = -0.01f;
            segment2.ZSpeed = 0.0f;

        }

        private void Render()
        {
            device.Clear(ClearFlags.Target, Color.White, 1.0f, 0);
            device.BeginScene();

            segment1.Draw(device);
            segment2.Draw(device);

            device.EndScene();
            device.Present();
        }


        /// <summary>
        /// Plays the game. Sets gameActive to true and
        /// updates the game until it becomes false.
        /// </summary>
        private void playGame()
        {
            gameActive = true;
            while (gameActive)
            {
                startTimer();
                if (!gamePaused)
                {
                    segment1.Update();
                    segment2.Update();
                }
                Application.DoEvents();
                Render();
                showTimer();
                System.Threading.Thread.Sleep(100);
            }
        }

        /// <summary>
        /// Refers to a method with the signature required for 
        /// Form.Invoke calls.
        /// </summary>
        private delegate void gameMethod();

        /// <summary>
        /// Refers to the method which will play the game. Set
        /// to playGame when the game starts.
        /// </summary>
        gameMethod playDelegate;

        /// <summary>
        /// Starts a new game.
        /// </summary>
        private void startGame()
        {
            playDelegate = new gameMethod(playGame);
            Invoke(playDelegate);
        }

        #endregion

        #region Event Handlers

        protected override void OnPaintBackground(PaintEventArgs e)
        {
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            Render();
        }

        #endregion 

        private void playMenuItem_Click(object sender, EventArgs e)
        {
            startGame();
        }
    }
}